@@ -1,4 +1,3 @@
-from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
 from django.db import models
@@ -6,22 +5,28 @@ from django.conf import settings
 from django.db import transaction
 import datetime
 
+from simple_import.compat import AUTH_USER_MODEL
+
+import sys
+if sys.version_info >= (3,0):
+    unicode = str
+
 class ImportSetting(models.Model):
     """ Save some settings per user per content type """
-    user = models.ForeignKey(User)
+    user = models.ForeignKey(AUTH_USER_MODEL)
     content_type = models.ForeignKey(ContentType)
     
     class Meta():
         unique_together = ('user', 'content_type',)
-    
-    
+
+
 class ColumnMatch(models.Model):
     """ Match column names from the user uploaded file to the database """
     column_name = models.CharField(max_length=200)
     field_name = models.CharField(max_length=255, blank=True)
     import_setting = models.ForeignKey(ImportSetting)
     default_value = models.CharField(max_length=2000, blank=True)
-    null_on_empty = models.BooleanField(help_text="If cell is blank, clear out the field setting it to blank.")
+    null_on_empty = models.BooleanField(default=False, help_text="If cell is blank, clear out the field setting it to blank.")
     header_position = models.IntegerField(help_text="Annoying way to order the columns to match the header rows")
     
     class Meta:
@@ -51,13 +56,12 @@ class ColumnMatch(models.Model):
             if hasattr(field, 'verbose_name'):
                 if field.verbose_name.lower().replace(' ', '_') == normalized_field_name:
                     self.field_name = field_name
-            
 
-    
+
 class ImportLog(models.Model):
     """ A log of all import attempts """
     name = models.CharField(max_length=255)
-    user = models.ForeignKey(User, editable=False, related_name="simple_import_log")
+    user = models.ForeignKey(AUTH_USER_MODEL, editable=False, related_name="simple_import_log")
     date = models.DateTimeField(auto_now_add=True, verbose_name="Date Created")
     import_file = models.FileField(upload_to="import_file")
     error_file = models.FileField(upload_to="error_file", blank=True)
@@ -92,33 +96,42 @@ class ImportLog(models.Model):
         """ Get each matching header row to database match
         Returns a ColumnMatch queryset"""
         header_row = self.get_import_file_as_list(only_header=True)
-        matches = ColumnMatch.objects.none()
         match_ids = []
+        
         for i, cell in enumerate(header_row):
-            if cell: # Sometimes we get blank headers, ignore them.
-                try:
-                    match = ColumnMatch.objects.get(
-                        import_setting = self.import_setting,
-                        column_name = cell,
-                    )
-                except ColumnMatch.DoesNotExist:
-                    match = ColumnMatch(
-                        import_setting = self.import_setting,
-                        column_name = cell,
-                    )
-                    match.guess_field()
-                match.header_position = i
-                match.save()
-                match_ids += [match.id]
+            if not cell: # Sometimes we get blank headers, ignore them.
+                continue
+            
+            try:
+                match = ColumnMatch.objects.get(
+                    import_setting = self.import_setting,
+                    column_name = cell,
+                )
+            except ColumnMatch.DoesNotExist:
+                match = ColumnMatch(
+                    import_setting = self.import_setting,
+                    column_name = cell,
+                )
+                match.guess_field()
+            
+            match.header_position = i
+            match.save()
+            
+            match_ids += [match.id]
+        
         return ColumnMatch.objects.filter(pk__in=match_ids).order_by('header_position')
 
     def get_import_file_as_list(self, only_header=False):
         file_ext = str(self.import_file).lower()[-3:]
         data = []
+        
+        self.import_file.seek(0)
+        
         if file_ext == "xls":
             import xlrd
             import os
-            wb = xlrd.open_workbook(os.path.join(settings.MEDIA_ROOT, self.import_file.file.name))
+            
+            wb = xlrd.open_workbook(file_contents=self.import_file.read())
             sh1 = wb.sheet_by_index(0)
             for rownum in range(sh1.nrows):
                 row_values = []
@@ -133,14 +146,15 @@ class ImportLog(models.Model):
                     break
         elif file_ext == "csv":
             import csv
-            reader = csv.reader(open(self.import_file.path, "rb"))
+            reader = csv.reader(self.import_file)
             for row in reader:
                 data += [row]
                 if only_header:
                     break
         elif file_ext == "lsx":
             from openpyxl.reader.excel import load_workbook
-            wb = load_workbook(filename=self.import_file.path, use_iterators = True)
+            # load_workbook actually accepts a file-like object for the filename param
+            wb = load_workbook(filename=self.import_file, use_iterators = True)
             sheet = wb.get_active_sheet()
             for row in sheet.iter_rows():
                 data_row = []
@@ -150,8 +164,8 @@ class ImportLog(models.Model):
                 if only_header:
                     break
         elif file_ext == "ods":
-            from odsreader import ODSReader
-            doc = ODSReader(self.import_file.path)
+            from .odsreader import ODSReader
+            doc = ODSReader(self.import_file)
             table = doc.SHEETS.items()[0]
 
             # Remove blank columns that ods files seems to have
@@ -197,9 +211,10 @@ class RelationalMatch(models.Model):
     field_name = models.CharField(max_length=255) # Ex student_number_set
     related_field_name = models.CharField(max_length=255, blank=True) # Ex username
 
-    
+
 class ImportedObject(models.Model):
     import_log = models.ForeignKey(ImportLog)
     object_id = models.IntegerField()
     content_type = models.ForeignKey(ContentType)
     content_object = generic.GenericForeignKey('content_type', 'object_id')
+
